"""Global suite of tests"""
import unittest

from veliberator.models import db_connection

from tests.test_popchecker import suite as popchecker_suite
from tests.test_models import suite as models_suite
from tests.test_xml_wrappers import suite as xml_suite
from tests.test_station import suite as station_suite
from tests.test_cartography import suite as cartography_suite

db_connection('sqlite:///:memory:')

global_test_suite = unittest.TestSuite([popchecker_suite,
                                        models_suite,
                                        xml_suite,
                                        station_suite,
                                        cartography_suite,])





#!/usr/bin/python
import sys, os

sys.path.append(os.path.abspath('.'))

from veliberator.models import db_connection
from veliberator.cartography import Cartography

if __name__ == '__main__':
    db_connection()
    print 'Database open, now synchronizing...'
    Cartography.synchronize()
    print 'Synchronization completed !'
    

    
    
    



"""Settings for Veliberator"""

MAIL_SSL = True
MAIL_PORT = MAIL_SSL and 995 or 110
MAIL_SERVER = 'pop.gmail.com'
MAIL_ADDRESS = 'fantomas42@gmail.com'
MAIL_ACCOUNT = MAIL_ADDRESS
MAIL_PASSWORD = '.4il1>Y$'

VELIB_DATA_XML_URL = 'http://www.velib.paris.fr/service/carto'
VELIB_STATUS_XML_URL = 'http://www.velib.paris.fr/service/stationdetails/%s'

DATABASE_URI = 'sqlite:///db.sqlite'
DATABASE_ECHO = False

TEST_XML_CARTOGRAPHY = 'tests/data/carto_short.xml'



"""Station objects for veliberator"""
from math import sqrt
from urllib import urlopen
from xml.dom.minidom import parse

from veliberator.models import StationInformation
from veliberator.settings import VELIB_STATUS_XML_URL
from veliberator.xml_wrappers import xml_station_status_wrapper

class UnknowStation(Exception):
    pass

class Station(object):
    """Station object"""

    def __init__(self, velib_id):
        self.id = int(velib_id)
        self.status = None
        self.informations = StationInformation.get(self.id)

        if not self.informations:
            raise UnknowStation('The Station ID does not exist.')

    def __repr__(self):
        if self.informations:
            return '<Station "%s" (%s)>' % (self.id, self.informations.address)
        return '<Station "%s">' % self.id
        
    def get_status(self):
        dom = parse(urlopen(VELIB_STATUS_XML_URL % self.id))        
        self.status = xml_station_status_wrapper(dom.firstChild)

    def show_status(self):
        if not self.status:
            self.get_status()
        print "Station '%s'" % self.id
        print '%s, %s %s' % (self.informations.address,
                             self.informations.postal_code,
                             self.informations.city)
        print '%s/%s velo(s) disponible' % (self.status['available'], self.status['total'])
        print '%s place(s) disponible' % self.status['free']

    def compute_distances(self, stations):
        distances = {}

        for station in stations:
            if station.id == self.id:
                continue
            l1 = float(self.informations.lat) - float(station.lat)
            l2 = float(self.informations.lng) - float(station.lng) 
            dist = sqrt(pow(l1, 2) + pow(l2, 2))
            distances[station] = dist

        return distances
        
    def get_stations_around(self, number=5):
        lat_orig = float(self.informations.lat)
        lat_pos = '%.2f' % (lat_orig + 0.01)
        lat_neg = '%.2f' % (lat_orig - 0.01)
        lat_orig = '%.2f' % lat_orig

        lng_orig = float(self.informations.lng)
        lng_pos = '%.2f' % (lng_orig + 0.01)
        lng_neg = '%.2f' % (lng_orig - 0.01)
        lng_orig = '%.2f' % lng_orig


        stations = StationInformation.query.filter(
            (StationInformation.lat.startswith(lat_orig) |
             StationInformation.lat.startswith(lat_pos) |
             StationInformation.lat.startswith(lat_neg)) &
            (StationInformation.lng.startswith(lng_orig) |
             StationInformation.lng.startswith(lng_pos) |
             StationInformation.lng.startswith(lng_neg))).all()

        station_distances = self.compute_distances(stations)
        station_distances_ordered = sorted(station_distances.iteritems(),
                                           key=lambda (k,v): (v,k))
        stations_id = [station.id for station, distance in station_distances_ordered]
        
        return stations_id[:number]
        



"""Unit tests for Station object"""
import unittest

from veliberator.settings import TEST_XML_CARTOGRAPHY
from veliberator.station import UnknowStation, Station
from veliberator.models import StationInformation
from veliberator.cartography import Cartography

class StationTestCase(unittest.TestCase):

    def setUp(self):
        self.velib_id = 42008
        self.informations = StationInformation(id=self.velib_id,
                                               address='Test')
        self.informations.save()

    def tearDown(self):
        self.informations = None
        StationInformation.query.delete()

    def test_InformationsErrorRaising(self):
        self.assertRaises(UnknowStation, Station, 0)

    def test_InformationsNoError(self):
        station = Station(self.velib_id)
        self.assertTrue(isinstance(station.informations, StationInformation))
        station = Station(str(self.velib_id))
        self.assertTrue(isinstance(station.informations, StationInformation))
        

    def test_GetStatus(self):
        station = Station(self.velib_id)
        self.assertEquals(station.status, None)

        station.get_status()
        self.assertTrue(isinstance(station.status, dict))

    def test_ComputeDistances(self):
        station = Station(self.velib_id)
        result = station.compute_distances([station.informations])
        self.assertEquals(result, {})

        station.informations.lat = 1
        station.informations.lng = 1
        si = StationInformation(id=42, lat=4, lng=5)
        
        result = station.compute_distances([si])
        self.assertEquals(result, {si: 5.0})

    def test_GetStationsAround(self):
        Cartography.synchronize(TEST_XML_CARTOGRAPHY)
        
        station = Station(self.velib_id)

        stations_around = station.get_stations_around(1)
        self.assertEquals(len(stations_around), 1)

        stations_around = station.get_stations_around(5)
        self.assertEquals(len(stations_around), 5)
        self.assertEquals(stations_around, [42006, 42010, 42012, 42014, 42016])

        Cartography.flush()


suite = unittest.TestLoader().loadTestsFromTestCase(StationTestCase)



#!/usr/bin/python
import sys, os

sys.path.append(os.path.abspath('.'))

from veliberator.station import Station
from veliberator.models import db_connection

if __name__ == '__main__':
    db_connection()
    station = Station(sys.argv[1])
    station.show_status()
                        
    if not station.status['free']:
        print 'Pas de places disponibles...'
        print 'Calcul des places les plus proches.'
        
        for station_id in station.get_stations_around(3):
            station = Station(station_id)
            station.get_status()
            if station.status['free']:
                print '----------'
                station.show_status()
    



"""Pop checker waiting for actions"""
from poplib import POP3, POP3_SSL

from veliberator.settings import *

pop_checker_class = MAIL_SSL and POP3_SSL or POP3

class PopChecker(pop_checker_class):
    """object for checking messages on POP server"""
    
    def __init__(self, server=MAIL_SERVER, port=MAIL_PORT,
                 login=MAIL_ACCOUNT, password=MAIL_PASSWORD):
        self.server = server
        self.port = port
        self.login = login
        self.password = password
        pop_checker_class.__init__(self, self.server, self.port)

    def authenticate(self):
        self.user(self.login)
        self.pass_(self.password)

    


pop_checker = PopChecker()









"""Installing veliberator"""
from setuptools import setup, find_packages
import sys, os

version = '0.1.2'

setup(
    name='veliberator',
    version=version,
    zip_safe=True,

    scripts=['./bin/find_place.py',
             './bin/synchronize.py'],
    packages=find_packages(exclude=['tests',]),
    install_requires = ['SQLAlchemy>=0.5',
                        'Elixir>=0.6.1',],
    include_package_data=True,
        
    test_suite = 'tests.global_test_suite',

    author='Fantomas42',
    author_email='fantomas42@gmail.com',
    url='http://fantomas.willbreak.it',
 
    license='GPL',
    platforms = 'any',
    description="Python API for Velib.",
    long_description=open(os.path.join("docs", "README.txt")).read(),
    keywords='velib, api, service',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    )



"""Unit tests for PopChecker object"""
import unittest

from veliberator.popchecker import PopChecker

class PopCheckerTestCase(unittest.TestCase):

    def __test_ConfigPopChecker(self):
        pop_checker = PopChecker()
        pop_checker.authentificate()

    def __test_Authentificate(self):
        pop_checker = PopChecker('bla.gog', 45)
        pop_checker.authentificate()
        
        self.assertEquals(1, 1)

    

suite = unittest.TestLoader().loadTestsFromTestCase(PopCheckerTestCase)
